package com.loganmagnan.hubcore;

import aether.Aether;
import com.loganmagnan.hubcore.managers.PlayerDataManager;
import com.loganmagnan.hubcore.managers.SpawnManager;
import com.loganmagnan.hubcore.managers.cosmetic.CosmeticManager;
import com.loganmagnan.hubcore.managers.item.HotBarManager;
import com.loganmagnan.hubcore.menusystem.PlayerMenuUtil;
import com.loganmagnan.hubcore.playerdata.PlayerData;
import com.loganmagnan.hubcore.scoreboard.ScoreboardProvider;
import com.loganmagnan.hubcore.scoreboard.ScoreboardRunnable;
import com.loganmagnan.hubcore.scoreboard.SimpleBoard;
import com.loganmagnan.hubcore.utils.BungeeUtils;
import com.loganmagnan.hubcore.utils.command.CommandFramework;
import com.loganmagnan.hubcore.utils.ClassRegistrationUtils;
import com.loganmagnan.hubcore.utils.ColorUtils;
import com.loganmagnan.hubcore.utils.Utils;
import com.loganmagnan.hubcore.utils.config.FileConfig;
import com.loganmagnan.hubcore.utils.config.file.Config;
import lombok.Getter;
import lombok.Setter;
import net.milkbowl.vault.chat.Chat;
import org.bukkit.Bukkit;
import org.bukkit.Chunk;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.permissions.Permission;
import org.bukkit.plugin.RegisteredServiceProvider;
import org.bukkit.plugin.java.JavaPlugin;

import java.util.HashMap;

@Getter
@Setter
public class HubCore extends JavaPlugin {

    @Getter private static HubCore instance;

    private Config mainConfig;
    private FileConfig messagesConfig;
    private FileConfig scoreboardConfig;
    private FileConfig hotBarConfig;
    private FileConfig menusConfig;
    private FileConfig cosmeticsConfig;

    private PlayerDataManager playerDataManager;
    private HotBarManager hotBarManager;
    private SpawnManager spawnManager;
    private CosmeticManager cosmeticManager;

    private HashMap<Player, PlayerMenuUtil> playerMenuUtilMap = new HashMap<>();

    private CommandFramework commandFramework = new CommandFramework(this);

    private Chat chat;
    private Permission perms;

    @Override
    public void onEnable() {
        instance = this;

        this.saveDefaultConfig();
        this.mainConfig = new Config("config", this);
        this.messagesConfig = new FileConfig(this, "messages.yml");
        this.scoreboardConfig = new FileConfig(this, "scoreboard.yml");
        this.hotBarConfig = new FileConfig(this, "hotbar.yml");
        this.menusConfig = new FileConfig(this, "menus.yml");
        this.cosmeticsConfig = new FileConfig(this, "cosmetics.yml");

        Bukkit.getConsoleSender().sendMessage(Utils.chatBar);
        Bukkit.getConsoleSender().sendMessage(ColorUtils.getMessageType("&dHubCore &7- &av" + this.getDescription().getVersion()));
        Bukkit.getConsoleSender().sendMessage(ColorUtils.getMessageType("&7Made by &eLoganM Development"));
        Bukkit.getConsoleSender().sendMessage(Utils.chatBar);

        this.getServer().getMessenger().registerOutgoingPluginChannel(this, "BungeeCord");
        this.getServer().getMessenger().registerIncomingPluginChannel(this, "BungeeCord", new BungeeUtils());
        this.loadCommands();
        this.loadManagers();
        this.loadListeners();
        this.loadRunnables();

        for (World world : Bukkit.getWorlds()) {
            for (Entity entity : world.getEntities()) {
                if (entity.getType() != EntityType.PLAYER && entity.getType() != EntityType.ITEM_FRAME) {
                    entity.remove();
                }
            }

            world.setGameRuleValue("doDaylightCycle", "false");
            world.setTime(0L);
            world.setStorm(false);
        }

        if (this.getServer().getPluginManager().getPlugin("Vault") != null) {
            this.setupChat();
            this.setupPermissions();
        }
    }

    @Override
    public void onDisable() {
        for (Player player : this.getServer().getOnlinePlayers()) {
            PlayerData playerData = this.playerDataManager.getPlayerData(player.getUniqueId());

            for (Block block : playerData.getBlocks()) {
                if (block.getType() == Material.AIR) {
                    continue;
                }

                block.setType(Material.AIR);
            }

            this.playerDataManager.savePlayerData(playerData);
        }

        for (World world : Bukkit.getWorlds()) {
            for (Entity entity : world.getEntities()) {
                if (entity.getType() == EntityType.DROPPED_ITEM) {
                    entity.remove();
                }
            }

            for (Chunk chunk : world.getLoadedChunks()) {
                chunk.unload(true);
            }
        }

        instance = null;
    }

    public void loadCommands() {
        ClassRegistrationUtils.loadCommands("com.loganmagnan.hubcore.commands");
    }

    public void loadManagers() {
        this.playerDataManager = new PlayerDataManager();
        this.hotBarManager = new HotBarManager();
        this.spawnManager = new SpawnManager();
        this.cosmeticManager = new CosmeticManager();
    }

    public void loadListeners() {
        ClassRegistrationUtils.loadListeners("com.loganmagnan.hubcore.listeners");
    }

    public void loadRunnables() {
//        new Aether(this, new ScoreboardProvider());
        new ScoreboardRunnable().runTaskTimer(this, 0, 20);
    }

    public PlayerMenuUtil getPlayerMenuUtil(Player player) {
        PlayerMenuUtil playerMenuUtil;

        if (playerMenuUtilMap.containsKey(player)) {
            return playerMenuUtilMap.get(player);
        } else {
            playerMenuUtil = new PlayerMenuUtil(player);

            playerMenuUtilMap.put(player, playerMenuUtil);

            return playerMenuUtil;
        }
    }

    public boolean setupChat() {
        RegisteredServiceProvider<Chat> chatProvider = getServer().getServicesManager().getRegistration(Chat.class);

        if (chatProvider != null) {
            chat = chatProvider.getProvider();
        }

        return (chat != null);
    }

    public boolean setupPermissions() {
        RegisteredServiceProvider<Permission> permissionProvider = getServer().getServicesManager().getRegistration(Permission.class);

        if (permissionProvider != null) {
            perms = permissionProvider.getProvider();
        }

        return (perms != null);
    }
}
