package com.loganmagnan.hubcore.listeners;

import com.loganmagnan.hubcore.HubCore;
import com.loganmagnan.hubcore.playerdata.PlayerData;
import com.loganmagnan.hubcore.utils.ColorUtils;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.*;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;

public class PlayerDataListener implements Listener {

	private HubCore main = HubCore.getInstance();

	@EventHandler(priority = EventPriority.HIGHEST, ignoreCancelled = true)
	public void onAsyncPlayerPreLogin(AsyncPlayerPreLoginEvent event) {
		Player player = Bukkit.getPlayer(event.getUniqueId());

		if (player != null) {
			if (player.isOnline()) {
				event.setLoginResult(AsyncPlayerPreLoginEvent.Result.KICK_OTHER);
				event.setKickMessage("§cYou tried to login too quickly after disconnecting.\n§cTry again in a few seconds.");

				this.main.getServer().getScheduler().runTask(this.main, () -> player.kickPlayer("§cDuplicate Login"));

				return;
			}

			PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());
		}
	}

	@EventHandler(priority = EventPriority.LOWEST, ignoreCancelled = true)
	public void onPlayerLogin(PlayerLoginEvent event) {
		PlayerData playerData = this.main.getPlayerDataManager().getOrCreate(event.getPlayer().getUniqueId());

		if (playerData == null) {
			event.setResult(PlayerLoginEvent.Result.KICK_OTHER);
			event.setKickMessage("§cAn error has occurred while loading your profile. Please reconnect.");

			return;
		}

		if (!playerData.isLoaded()) {
			event.setResult(PlayerLoginEvent.Result.KICK_OTHER);
			event.setKickMessage("§cAn error has occurred while loading your profile. Please reconnect.");
		}
	}

	@EventHandler
	public void onPlayerJoin(PlayerJoinEvent event) {
		Player player = event.getPlayer();

		this.main.getPlayerDataManager().sendToSpawnAndResetPlayer(player);
		this.main.getSpawnManager().getPlayers().add(player);

		player.addPotionEffect(new PotionEffect(PotionEffectType.SPEED, Integer.MAX_VALUE, 4));

		event.setJoinMessage(null);

		for (String string : main.getMessagesConfig().getConfig().getStringList("JOIN-MESSAGE")) {
			player.sendMessage(ColorUtils.getMessageType(string.replace("%player%", player.getName())));
		}
	}

	@EventHandler
	public void onPlayerQuit(PlayerQuitEvent event) {
		event.setQuitMessage(null);

		Player player = event.getPlayer();

		PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

		this.handleLeave(player);
		this.handleDataSave(playerData);
		this.main.getSpawnManager().getPlayers().remove(player);
	}

	@EventHandler
	public void onPlayerKick(PlayerKickEvent event) {
		event.setLeaveMessage(null);

		Player player = event.getPlayer();

		PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

		this.handleLeave(player);
		this.handleDataSave(playerData);
		this.main.getSpawnManager().getPlayers().remove(player);
	}

	private void handleLeave(Player player) {

	}

	private void handleDataSave(PlayerData playerData) {
		if (playerData != null) {
			this.main.getPlayerDataManager().deletePlayer(playerData.getUniqueId());
		}
	}
}
