package com.loganmagnan.hubcore.scoreboard;

import com.loganmagnan.hubcore.HubCore;
import com.loganmagnan.hubcore.playerdata.PlayerData;
import com.loganmagnan.hubcore.playerdata.PlayerSettings;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ScoreboardRunnable extends BukkitRunnable {

    private HubCore main = HubCore.getInstance();

    private Map<UUID, SimpleBoard> scoreboards = new HashMap<UUID, SimpleBoard>();

    private Pattern HEX_PATTERN = Pattern.compile("&#([A-Fa-f0-9]{6})");

    @Override
    public void run() {
        for (Player player : this.main.getSpawnManager().getPlayers()) {
            SimpleBoard simpleBoard = null;

            if (this.scoreboards.get(player.getUniqueId()) == null) {
                simpleBoard = new SimpleBoard();

                this.scoreboards.put(player.getUniqueId(), simpleBoard);
            } else {
                simpleBoard = this.scoreboards.get(player.getUniqueId());
            }

            simpleBoard.setTitle(colorize(translateHexColorCodes(this.main.getScoreboardConfig().getConfig().getString("SCOREBOARD.TITLE"))));

            PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

            if (playerData == null) {
                return;
            }

            PlayerSettings playerSettings = playerData.getPlayerSettings();

            if (!playerSettings.isScoreboardEnabled()) {
                simpleBoard.displayList(new ArrayList<String>());
                simpleBoard.display(player);

                return;
            }

            switch (playerData.getPlayerState()) {
                case SPAWN:
                    simpleBoard.displayList(simpleBoard.getSpawnScoreboard(player, playerData));
                    simpleBoard.display(player);

                    break;
            }
        }
    }

    public String colorize(String message) {
        return ChatColor.translateAlternateColorCodes('&', message);
    }

    public String translateHexColorCodes(String message) {
        char colorChar = '§';
        Matcher matcher = HEX_PATTERN.matcher(message);
        StringBuffer buffer = new StringBuffer(message.length() + 32);
        while (matcher.find()) {
            String group = matcher.group(1);
            matcher.appendReplacement(buffer, colorChar + "x" + colorChar + group
                    .charAt(0) + colorChar + group.charAt(1) + colorChar + group
                    .charAt(2) + colorChar + group.charAt(3) + colorChar + group
                    .charAt(4) + colorChar + group.charAt(5));
        }
        return matcher.appendTail(buffer).toString();
    }
}
